from __future__ import print_function
import numpy as np
import healpy as hp
import matplotlib.pyplot as plt
from astropy.wcs import WCS
from astropy.io import fits
import pymaster as nmt

DTOR=np.pi/180

l,cltt,clee,clbb,clte,nltt,nlee,nlbb,nlte=np.loadtxt("cls_lss.txt",unpack=True)


def write_flat_map(filename,maps,wcs,descript=None) :
    if maps.ndim<2 :
        raise ValueError("Must supply at least one map")
    if maps.ndim==2 :
        maps=np.array([maps])
    if descript is not None :
        if len(maps)==1 :
            descript=[descript]
        if len(maps)!=len(descript) :
            raise ValueError("Need one description per map")

    header=wcs.to_header()
    hdus=[]
    for im,m in enumerate(maps) :
        head=header.copy()
        if m.shape!=(wcs._naxis2,wcs._naxis1) :
            raise ValueError("Map shape differs from WCS!")
        if descript is not None :
            head['DESCR']=(descript[im],'Description')
        if im==0 :
            hdu=fits.PrimaryHDU(data=m,header=head)
        else :
            hdu=fits.ImageHDU(data=m,header=head)
        hdus.append(hdu)
    hdulist=fits.HDUList(hdus)
    hdulist.writeto(filename,overwrite=True)

def read_flat_map(filename,i_map=0) :
    """
    Reads a flat-sky map and the details of its pixelization scheme.
    The latter are returned as a FlatMapInfo object.
    i_map : map to read. If -1, all maps will be read.
    """
    hdul=fits.open(filename)
    w=WCS(hdul[0].header)

    maps=hdul[i_map].data
    ny,nx=maps.shape

    return w,maps

##########################
# CAR stuff

def getmaskapoana_car(w,aps,fsk=0.1,dec0=-50,ra0=0.) :
    v0=np.array([np.sin(DTOR*(90-dec0))*np.cos(DTOR*ra0),
                 np.sin(DTOR*(90-dec0))*np.sin(DTOR*ra0),
                 np.cos(DTOR*(90-dec0))])
    ph,th=w.wcs_pix2world(np.array(np.meshgrid(np.arange(w._naxis1),np.arange(w._naxis2))).reshape([2,w._naxis2*w._naxis1]).T,0).T.reshape([2,w._naxis2,w._naxis1])
    ph=np.radians(ph)
    th=np.radians(90-th)
    vv=np.array([np.sin(th)*np.cos(ph),np.sin(th)*np.sin(ph),np.cos(th)])
    cth=np.sum(v0[:,None,None]*vv,axis=0); th=np.arccos(cth).flatten();
    th0=np.arccos(1-2*fsk); th_apo=aps*DTOR
    id0=np.where(th.flatten()>=th0)[0]
    id1=np.where(th<=th0-th_apo)[0]
    idb=np.where((th>th0-th_apo) & (th<th0))[0]
    x=np.sqrt((1-np.cos(th[idb]-th0))/(1-np.cos(th_apo)))
    mask_apo=np.zeros(w._naxis2*w._naxis1)
    mask_apo[id0]=0.
    mask_apo[id1]=1.
    mask_apo[idb]=x-np.sin(2*np.pi*x)/(2*np.pi)
    mask_apo=mask_apo.reshape([w._naxis2,w._naxis1])
    return mask_apo

dra=1.; ddec=-1.; nx=360; ny=181;
wcs=WCS(naxis=2)
wcs.wcs.cdelt=[dra,ddec]
wcs.wcs.crval=[0,0]
wcs.wcs.ctype=['RA---CAR','DEC--CAR']
wcs.wcs.crpix=[1+180/dra,1+90/np.fabs(ddec)]
dl,dw_q,dw_u=nmt.synfast_spherical(-1,[cltt+nltt,clte+nlte,0*cltt,clee+nlee,0*clee,clbb+nlbb],[0,2],wcs=wcs)
sl,sw_q,sw_u=nmt.synfast_spherical(-1,[cltt/(l+1.)**1.5,0*clte,0*cltt,clee/(l+1.)**1.5,0*clee,0.5*clee/(l+1.)**1.5],[0,2],wcs=wcs)
wcs._naxis2,wcs._naxis1=dl.shape

mask=getmaskapoana_car(wcs,20.,0.4,dec0=90.)
#write_flat_map("mps_car_small.fits",np.array([dl,dw_q,dw_u]),wcs,["T","Q","U"])
#write_flat_map("tmp_car_small.fits",np.array([sl,sw_q,sw_u]),wcs,["T","Q","U"])
#write_flat_map("msk_car_small.fits",mask,wcs,"mask")


##########################
# Flat-sky stuff
wcs,msk=read_flat_map("msk_flat.fits")
(ny,nx)=msk.shape
lx=np.fabs(nx*wcs.wcs.cdelt[0])*np.pi/180
ly=np.fabs(ny*wcs.wcs.cdelt[1])*np.pi/180
dt,dq,du=nmt.synfast_flat(int(nx),int(ny),lx,ly,
                          [cltt+nltt,clte+nlte,0*cltt,clee+nlee,0*clee,clbb+nlbb],[0,2])
write_flat_map("mps_flat.fits",np.array([dt,dq,du]),wcs,["T","Q","U"])
d_ell=20; lmax=500.; ledges=np.arange(int(lmax/d_ell)+1)*d_ell+2
_,st=read_flat_map("tmp_flat.fits",0)
_,sq=read_flat_map("tmp_flat.fits",1)
_,su=read_flat_map("tmp_flat.fits",2)

b=nmt.NmtBinFlat(ledges[:-1],ledges[1:])
leff=b.get_effective_ells();

#No contaminants
prefix='bm_f_nc_np'
f0=nmt.NmtFieldFlat(lx,ly,msk,[dt])
f2=nmt.NmtFieldFlat(lx,ly,msk,[dq,du])
w00=nmt.NmtWorkspaceFlat(); w00.compute_coupling_matrix(f0,f0,b);
cw00=nmt.NmtCovarianceWorkspaceFlat(); cw00.compute_coupling_coefficients(f0,f0,b);
cw00.write_to(prefix+'_cw00.fits')
cov=nmt.gaussian_covariance_flat(cw00,0,0,0,0,l,
                                 [cltt+nltt],[cltt+nltt],[cltt+nltt],[cltt+nltt],
                                 w00);
np.savetxt(prefix+"_cov.txt",cov)
clb00=w00.couple_cell(l,np.array([nltt]))
c00=w00.decouple_cell(nmt.compute_coupled_cell_flat(f0,f0,b),cl_bias=clb00)
w00.write_to(prefix+'_w00.fits')
np.savetxt(prefix+'_c00.txt',np.transpose([leff,c00[0]]));
w02=nmt.NmtWorkspaceFlat(); w02.compute_coupling_matrix(f0,f2,b);
clb02=w02.couple_cell(l,np.array([nlte,0*nlte]))
c02=w02.decouple_cell(nmt.compute_coupled_cell_flat(f0,f2,b),cl_bias=clb02)
w02.write_to(prefix+'_w02.fits')
np.savetxt(prefix+'_c02.txt',np.transpose([leff,c02[0],c02[1]]));
w22=nmt.NmtWorkspaceFlat(); w22.compute_coupling_matrix(f2,f2,b);
clb22=w22.couple_cell(l,np.array([nlee,0*nlee,0*nlbb,nlbb]))
c22=w22.decouple_cell(nmt.compute_coupled_cell_flat(f2,f2,b),cl_bias=clb22)
w22.write_to(prefix+'_w22.fits')
np.savetxt(prefix+'_c22.txt',np.transpose([leff,c22[0],c22[1],c22[2],c22[3]]));
# Covariance for 02,02
cov=nmt.gaussian_covariance_flat(cw00,0,2,0,2,l,
                                 [cltt+nltt],
                                 [clte, 0*clte],
                                 [clte, 0*clte],
                                 [clee+nlee,0*clee,0*clee,clbb+nlbb],
                                 w02, wb=w02);
np.savetxt(prefix+"_cov0202.txt",cov)
# Covariance for 00,02
cov=nmt.gaussian_covariance_flat(cw00,0,0,0,2,l,
                                 [cltt+nltt],
                                 [clte, 0*clte],
                                 [cltt+nltt],
                                 [clte, 0*clte],
                                 w00, wb=w02);
np.savetxt(prefix+"_cov0002.txt",cov)
# Covariance for 00,22
cov=nmt.gaussian_covariance_flat(cw00,0,0,2,2,l,
                                 [clte, 0*clte],
                                 [clte, 0*clte],
                                 [clte, 0*clte],
                                 [clte, 0*clte],
                                 w00, wb=w22);
np.savetxt(prefix+"_cov0022.txt",cov)
# Covariance for 22,22
cov=nmt.gaussian_covariance_flat(cw00,2,2,2,2,l,
                                 [clee+nlee, 0*clee, 0*clee, clbb+nlbb],
                                 [clee+nlee, 0*clee, 0*clee, clbb+nlbb],
                                 [clee+nlee, 0*clee, 0*clee, clbb+nlbb],
                                 [clee+nlee, 0*clee, 0*clee, clbb+nlbb],
                                 w22, wb=w22);
np.savetxt(prefix+"_cov2222.txt",cov)

#With contaminants
prefix='bm_f_yc_np'
f0=nmt.NmtFieldFlat(lx,ly,msk,[dt],[[st]])
f2=nmt.NmtFieldFlat(lx,ly,msk,[dq,du],[[sq,su]])
w00=nmt.NmtWorkspaceFlat(); w00.compute_coupling_matrix(f0,f0,b);
clb00=nmt.deprojection_bias_flat(f0,f0,b,l,[cltt+nltt])
np.savetxt(prefix+'_cb00.txt',np.transpose([leff,clb00[0]]))
clb00+=w00.couple_cell(l,np.array([nltt]))
c00=w00.decouple_cell(nmt.compute_coupled_cell_flat(f0,f0,b),cl_bias=clb00)
w00.write_to(prefix+'_w00.fits')
np.savetxt(prefix+'_c00.txt',np.transpose([leff,c00[0]]));
w02=nmt.NmtWorkspaceFlat(); w02.compute_coupling_matrix(f0,f2,b);
clb02=nmt.deprojection_bias_flat(f0,f2,b,l,[clte+nlte,0*clte])
np.savetxt(prefix+'_cb02.txt',np.transpose([leff,clb02[0],clb02[1]]))
clb02+=w02.couple_cell(l,np.array([nlte,0*nlte]))
c02=w02.decouple_cell(nmt.compute_coupled_cell_flat(f0,f2,b),cl_bias=clb02)
w02.write_to(prefix+'_w02.fits')
np.savetxt(prefix+'_c02.txt',np.transpose([leff,c02[0],c02[1]]));
w22=nmt.NmtWorkspaceFlat(); w22.compute_coupling_matrix(f2,f2,b);
clb22=nmt.deprojection_bias_flat(f2,f2,b,l,[clee+nlee,0*clee,0*clbb,clbb+nlbb])
np.savetxt(prefix+'_cb22.txt',np.transpose([leff,clb22[0],clb22[1],clb22[2],clb22[3]]))
clb22+=w22.couple_cell(l,np.array([nlee,0*nlee,0*nlbb,nlbb]))
c22=w22.decouple_cell(nmt.compute_coupled_cell_flat(f2,f2,b),cl_bias=clb22)
w22.write_to(prefix+'_w22.fits')
np.savetxt(prefix+'_c22.txt',np.transpose([leff,c22[0],c22[1],c22[2],c22[3]]));

#No contaminants, purified
prefix='bm_f_nc_yp'
f0=nmt.NmtFieldFlat(lx,ly,msk,[dt])
f2=nmt.NmtFieldFlat(lx,ly,msk,[dq,du],purify_b=True)
w00=nmt.NmtWorkspaceFlat(); w00.compute_coupling_matrix(f0,f0,b);
clb00=w00.couple_cell(l,np.array([nltt]))
c00=w00.decouple_cell(nmt.compute_coupled_cell_flat(f0,f0,b),cl_bias=clb00)
w00.write_to(prefix+'_w00.fits')
np.savetxt(prefix+'_c00.txt',np.transpose([leff,c00[0]]));
w02=nmt.NmtWorkspaceFlat(); w02.compute_coupling_matrix(f0,f2,b);
clb02=w02.couple_cell(l,np.array([nlte,0*nlte]))
c02=w02.decouple_cell(nmt.compute_coupled_cell_flat(f0,f2,b),cl_bias=clb02)
w02.write_to(prefix+'_w02.fits')
np.savetxt(prefix+'_c02.txt',np.transpose([leff,c02[0],c02[1]]));
w22=nmt.NmtWorkspaceFlat(); w22.compute_coupling_matrix(f2,f2,b);
clb22=w22.couple_cell(l,np.array([nlee,0*nlee,0*nlbb,nlbb]))
c22=w22.decouple_cell(nmt.compute_coupled_cell_flat(f2,f2,b),cl_bias=clb22)
w22.write_to(prefix+'_w22.fits')
np.savetxt(prefix+'_c22.txt',np.transpose([leff,c22[0],c22[1],c22[2],c22[3]]));

#With contaminants, purified
prefix='bm_f_yc_yp'
f0=nmt.NmtFieldFlat(lx,ly,msk,[dt],[[st]])
f2=nmt.NmtFieldFlat(lx,ly,msk,[dq,du],[[sq,su]],purify_b=True)
w00=nmt.NmtWorkspaceFlat(); w00.compute_coupling_matrix(f0,f0,b);
clb00=nmt.deprojection_bias_flat(f0,f0,b,l,[cltt+nltt])
np.savetxt(prefix+'_cb00.txt',np.transpose([leff,clb00[0]]))
clb00+=w00.couple_cell(l,np.array([nltt]))
c00=w00.decouple_cell(nmt.compute_coupled_cell_flat(f0,f0,b),cl_bias=clb00)
w00.write_to(prefix+'_w00.fits')
np.savetxt(prefix+'_c00.txt',np.transpose([leff,c00[0]]));
w02=nmt.NmtWorkspaceFlat(); w02.compute_coupling_matrix(f0,f2,b);
clb02=nmt.deprojection_bias_flat(f0,f2,b,l,[clte+nlte,0*clte])
np.savetxt(prefix+'_cb02.txt',np.transpose([leff,clb02[0],clb02[1]]))
clb02+=w02.couple_cell(l,np.array([nlte,0*nlte]))
c02=w02.decouple_cell(nmt.compute_coupled_cell_flat(f0,f2,b),cl_bias=clb02)
w02.write_to(prefix+'_w02.fits')
np.savetxt(prefix+'_c02.txt',np.transpose([leff,c02[0],c02[1]]));
w22=nmt.NmtWorkspaceFlat(); w22.compute_coupling_matrix(f2,f2,b);
clb22=nmt.deprojection_bias_flat(f2,f2,b,l,[clee+nlee,0*clee,0*clbb,clbb+nlbb])
np.savetxt(prefix+'_cb22.txt',np.transpose([leff,clb22[0],clb22[1],clb22[2],clb22[3]]))
clb22+=w22.couple_cell(l,np.array([nlee,0*nlee,0*nlbb,nlbb]))
c22=w22.decouple_cell(nmt.compute_coupled_cell_flat(f2,f2,b),cl_bias=clb22)
w22.write_to(prefix+'_w22.fits')
np.savetxt(prefix+'_c22.txt',np.transpose([leff,c22[0],c22[1],c22[2],c22[3]]));

def getmaskapoana(ns,aps,fsk=0.1,dec0=-50,ra0=0.) :
    """
    Generates a correctly-apodized mask
    """
    v0=np.array([np.sin(DTOR*(90-dec0))*np.cos(DTOR*ra0),
                 np.sin(DTOR*(90-dec0))*np.sin(DTOR*ra0),
                 np.cos(DTOR*(90-dec0))])
    vv=np.array(hp.pix2vec(ns,np.arange(hp.nside2npix(ns))))
    cth=np.sum(v0[:,None]*vv,axis=0); th=np.arccos(cth); th0=np.arccos(1-2*fsk); th_apo=aps*DTOR
    id0=np.where(th>=th0)[0]
    id1=np.where(th<=th0-th_apo)[0]
    idb=np.where((th>th0-th_apo) & (th<th0))[0]
    x=np.sqrt((1-np.cos(th[idb]-th0))/(1-np.cos(th_apo)))
    mask_apo=np.zeros(hp.nside2npix(ns))
    mask_apo[id0]=0.
    mask_apo[id1]=1.
    mask_apo[idb]=x-np.sin(2*np.pi*x)/(2*np.pi)
    return mask_apo


nside_out=64
mask=getmaskapoana(nside_out,20.,0.4,dec0=90.)
dl,dw_q,dw_u=hp.synfast([(cltt+nltt)[:3*nside_out],
                         (clee+nlee)[:3*nside_out],
                         (clbb+nlbb)[:3*nside_out],
                         (clte+nlte)[:3*nside_out]],
                        nside_out,new=True,verbose=False,pol=True)
sl=hp.ud_grade(hp.read_map("../../sandbox_validation/data/cont_lss_dust_ns64.fits",
                           field=0,verbose=False),nside_out=nside_out);
sw_q=hp.ud_grade(hp.read_map("../../sandbox_validation/data/cont_wl_psf_ns64.fits",
                             field=0,verbose=False),nside_out=nside_out);
sw_u=hp.ud_grade(hp.read_map("../../sandbox_validation/data/cont_wl_psf_ns64.fits",
                             field=1,verbose=False),nside_out=nside_out);
hp.write_map("msk.fits",mask,overwrite=True)
hp.write_map("mps.fits",[dl,dw_q,dw_u],overwrite=True)
hp.write_map("tmp.fits",[sl,sw_q,sw_u],overwrite=True)

b=nmt.NmtBin(nside_out,nlb=16)
leff=b.get_effective_ells()
lfull=np.arange(3*nside_out)

#No contaminants
prefix='bm_nc_np'
f0=nmt.NmtField(mask,[dl])
f2=nmt.NmtField(mask,[dw_q,dw_u])
w00=nmt.NmtWorkspace(); w00.compute_coupling_matrix(f0,f0,b);
cw00=nmt.NmtCovarianceWorkspace(); cw00.compute_coupling_coefficients(f0,f0);
cw00.write_to(prefix+'_cw00.fits')
cov=nmt.gaussian_covariance(cw00,0,0,0,0,
                            [(cltt+nltt)[:3*nside_out]],
                            [(cltt+nltt)[:3*nside_out]],
                            [(cltt+nltt)[:3*nside_out]],
                            [(cltt+nltt)[:3*nside_out]],
                            w00)
np.savetxt(prefix+"_cov.txt",cov)
clb00=np.array([nltt[:3*nside_out]])
c00=w00.decouple_cell(nmt.compute_coupled_cell(f0,f0),cl_bias=clb00)
w00.write_to(prefix+'_w00.fits')
np.savetxt(prefix+"_c00.txt",np.transpose([leff,c00[0]]))
w02=nmt.NmtWorkspace(); w02.compute_coupling_matrix(f0,f2,b);
clb02=np.array([nlte[:3*nside_out],0*nlte[:3*nside_out]])
c02=w02.decouple_cell(nmt.compute_coupled_cell(f0,f2),cl_bias=clb02)
w02.write_to(prefix+'_w02.fits')
np.savetxt(prefix+"_c02.txt",np.transpose([leff,c02[0],c02[1]]))
w22=nmt.NmtWorkspace(); w22.compute_coupling_matrix(f2,f2,b);
clb22=np.array([nlee[:3*nside_out],0*nlee[:3*nside_out],
                0*nlbb[:3*nside_out],nlbb[:3*nside_out]])
c22=w22.decouple_cell(nmt.compute_coupled_cell(f2,f2),cl_bias=clb22)
w22.write_to(prefix+'_w22.fits')
np.savetxt(prefix+"_c22.txt",np.transpose([leff,c22[0],c22[1],c22[2],c22[3]]))
# Covariance for 02,02
cov=nmt.gaussian_covariance(cw00,0,2,0,2,
                            [(cltt+nltt)[:3*nside_out]],
                            [clte[:3*nside_out], 0*clte[:3*nside_out]],
                            [clte[:3*nside_out], 0*clte[:3*nside_out]],
                            [(clee+nlee)[:3*nside_out],
                             0*clee[:3*nside_out],
                             0*clee[:3*nside_out],
                             (clbb+nlbb)[:3*nside_out]],
                            w02, wb=w02);
np.savetxt(prefix+"_cov0202.txt",cov)
# Covariance for 00,02
cov=nmt.gaussian_covariance(cw00,0,0,0,2,
                            [(cltt+nltt)[:3*nside_out]],
                            [clte[:3*nside_out], 0*clte[:3*nside_out]],
                            [(cltt+nltt)[:3*nside_out]],
                            [clte[:3*nside_out], 0*clte[:3*nside_out]],
                            w00, wb=w02);
np.savetxt(prefix+"_cov0002.txt",cov)
# Covariance for 00,22
cov=nmt.gaussian_covariance(cw00,0,0,2,2,
                            [clte[:3*nside_out], 0*clte[:3*nside_out]],
                            [clte[:3*nside_out], 0*clte[:3*nside_out]],
                            [clte[:3*nside_out], 0*clte[:3*nside_out]],
                            [clte[:3*nside_out], 0*clte[:3*nside_out]],
                            w00, wb=w22);
np.savetxt(prefix+"_cov0022.txt",cov)
# Covariance for 22,22
cov=nmt.gaussian_covariance(cw00,2,2,2,2,
                            [(clee+nlee)[:3*nside_out], 0*clee[:3*nside_out], 0*clee[:3*nside_out], (clbb+nlbb)[:3*nside_out]],
                            [(clee+nlee)[:3*nside_out], 0*clee[:3*nside_out], 0*clee[:3*nside_out], (clbb+nlbb)[:3*nside_out]],
                            [(clee+nlee)[:3*nside_out], 0*clee[:3*nside_out], 0*clee[:3*nside_out], (clbb+nlbb)[:3*nside_out]],
                            [(clee+nlee)[:3*nside_out], 0*clee[:3*nside_out], 0*clee[:3*nside_out], (clbb+nlbb)[:3*nside_out]],
                            w22, wb=w22);
np.savetxt(prefix+"_cov2222.txt",cov)

#With contaminants
prefix='bm_yc_np'
f0=nmt.NmtField(mask,[dl],templates=[[sl]])
f2=nmt.NmtField(mask,[dw_q,dw_u],templates=[[sw_q,sw_u]])
w00=nmt.NmtWorkspace(); w00.compute_coupling_matrix(f0,f0,b);
clb00=nmt.deprojection_bias(f0,f0,[(cltt+nltt)[:3*nside_out]])
np.savetxt(prefix+'_cb00.txt',np.transpose([lfull,clb00[0]]))
clb00+=np.array([nltt[:3*nside_out]])
c00=w00.decouple_cell(nmt.compute_coupled_cell(f0,f0),cl_bias=clb00)
w00.write_to(prefix+'_w00.fits')
np.savetxt(prefix+"_c00.txt",np.transpose([leff,c00[0]]))
w02=nmt.NmtWorkspace(); w02.compute_coupling_matrix(f0,f2,b);
clb02=nmt.deprojection_bias(f0,f2,[(clte+nlte)[:3*nside_out],0*clte[:3*nside_out]])
np.savetxt(prefix+'_cb02.txt',np.transpose([lfull,clb02[0],clb02[1]]))
clb02+=np.array([nlte[:3*nside_out],0*nlte[:3*nside_out]])
c02=w02.decouple_cell(nmt.compute_coupled_cell(f0,f2),cl_bias=clb02)
w02.write_to(prefix+'_w02.fits')
np.savetxt(prefix+"_c02.txt",np.transpose([leff,c02[0],c02[1]]))
w22=nmt.NmtWorkspace(); w22.compute_coupling_matrix(f2,f2,b);
clb22=nmt.deprojection_bias(f2,f2,[(clee+nlee)[:3*nside_out],0*clee[:3*nside_out],
                                   0*clbb[:3*nside_out],(clbb+nlbb)[:3*nside_out]])
np.savetxt(prefix+'_cb22.txt',np.transpose([lfull,clb22[0],clb22[1],clb22[2],clb22[3]]))
clb22+=np.array([nlee[:3*nside_out],0*nlee[:3*nside_out],
                 0*nlbb[:3*nside_out],nlbb[:3*nside_out]])
c22=w22.decouple_cell(nmt.compute_coupled_cell(f2,f2),cl_bias=clb22)
w22.write_to(prefix+'_w22.fits')
np.savetxt(prefix+"_c22.txt",np.transpose([leff,c22[0],c22[1],c22[2],c22[3]]))

#No contaminants, purified
prefix='bm_nc_yp'
f0=nmt.NmtField(mask,[dl])
f2=nmt.NmtField(mask,[dw_q,dw_u],purify_b=True)
w00=nmt.NmtWorkspace(); w00.compute_coupling_matrix(f0,f0,b);
clb00=np.array([nltt[:3*nside_out]])
c00=w00.decouple_cell(nmt.compute_coupled_cell(f0,f0),cl_bias=clb00)
w00.write_to(prefix+'_w00.fits')
np.savetxt(prefix+"_c00.txt",np.transpose([leff,c00[0]]))
w02=nmt.NmtWorkspace(); w02.compute_coupling_matrix(f0,f2,b);
clb02=np.array([nlte[:3*nside_out],0*nlte[:3*nside_out]])
c02=w02.decouple_cell(nmt.compute_coupled_cell(f0,f2),cl_bias=clb02)
w02.write_to(prefix+'_w02.fits')
np.savetxt(prefix+"_c02.txt",np.transpose([leff,c02[0],c02[1]]))
w22=nmt.NmtWorkspace(); w22.compute_coupling_matrix(f2,f2,b);
clb22=np.array([nlee[:3*nside_out],0*nlee[:3*nside_out],
                0*nlbb[:3*nside_out],nlbb[:3*nside_out]])
c22=w22.decouple_cell(nmt.compute_coupled_cell(f2,f2),cl_bias=clb22)
w22.write_to(prefix+'_w22.fits')
np.savetxt(prefix+"_c22.txt",np.transpose([leff,c22[0],c22[1],c22[2],c22[3]]))

#With contaminants, purified
prefix='bm_yc_yp'
f0=nmt.NmtField(mask,[dl],templates=[[sl]])
f2=nmt.NmtField(mask,[dw_q,dw_u],templates=[[sw_q,sw_u]],purify_b=True)
w00=nmt.NmtWorkspace(); w00.compute_coupling_matrix(f0,f0,b);
clb00=nmt.deprojection_bias(f0,f0,[(cltt+nltt)[:3*nside_out]])
np.savetxt(prefix+'_cb00.txt',np.transpose([lfull,clb00[0]]))
clb00+=np.array([nltt[:3*nside_out]])
c00=w00.decouple_cell(nmt.compute_coupled_cell(f0,f0),cl_bias=clb00)
w00.write_to(prefix+'_w00.fits')
np.savetxt(prefix+"_c00.txt",np.transpose([leff,c00[0]]))
w02=nmt.NmtWorkspace(); w02.compute_coupling_matrix(f0,f2,b);
clb02=nmt.deprojection_bias(f0,f2,[(clte+nlte)[:3*nside_out],0*clte[:3*nside_out]])
np.savetxt(prefix+'_cb02.txt',np.transpose([lfull,clb02[0],clb02[1]]))
clb02+=np.array([nlte[:3*nside_out],0*nlte[:3*nside_out]])
c02=w02.decouple_cell(nmt.compute_coupled_cell(f0,f2),cl_bias=clb02)
w02.write_to(prefix+'_w02.fits')
np.savetxt(prefix+"_c02.txt",np.transpose([leff,c02[0],c02[1]]))
w22=nmt.NmtWorkspace(); w22.compute_coupling_matrix(f2,f2,b);
clb22=nmt.deprojection_bias(f2,f2,[(clee+nlee)[:3*nside_out],0*clee[:3*nside_out],
                                   0*clbb[:3*nside_out],(clbb+nlbb)[:3*nside_out]])
np.savetxt(prefix+'_cb22.txt',np.transpose([lfull,clb22[0],clb22[1],clb22[2],clb22[3]]))
clb22+=np.array([nlee[:3*nside_out],0*nlee[:3*nside_out],
                 0*nlbb[:3*nside_out],nlbb[:3*nside_out]])
c22=w22.decouple_cell(nmt.compute_coupled_cell(f2,f2),cl_bias=clb22)
w22.write_to(prefix+'_w22.fits')
np.savetxt(prefix+"_c22.txt",np.transpose([leff,c22[0],c22[1],c22[2],c22[3]]))
